
/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
#if !defined( OEM_BUILD ) && !defined( HLDEMO_BUILD )

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"

#ifndef CLIENT_DLL
#define BOLT_AIR_VELOCITY       1536
#define BOLT_WATER_VELOCITY     512

// UNDONE: Save/restore this?  Don't forget to set classname and LINK_ENTITY_TO_CLASS()
// 
// OVERLOADS SOME ENTVARS:
//
// speed - the ideal magnitude of my velocity
class CCrossbowBolt : public CBaseEntity
{
	void Spawn( void );
	void Precache( void );
	int  Classify ( void );
        void ReturnThink( void );
	void EXPORT BubbleThink( void );
	void EXPORT BoltTouch( CBaseEntity *pOther );


        int m_iTrail;

public:
        CCrossbow *m_pLauncher;// pointer back to the launcher that fired me. 
        static CCrossbowBolt *BoltCreate( CCrossbow *pLauncher );
};
LINK_ENTITY_TO_CLASS( crossbow_bolt, CCrossbowBolt );

CCrossbowBolt *CCrossbowBolt::BoltCreate( CCrossbow *pLauncher )
{
	// Create a new entity with CCrossbowBolt private data
	CCrossbowBolt *pBolt = GetClassPtr( (CCrossbowBolt *)NULL );
        pBolt->Spawn();
	pBolt->pev->classname = MAKE_STRING("bolt");

        pBolt->m_pLauncher = pLauncher;// remember what RPG fired me. 
        pBolt->m_pLauncher->m_cActiveRockets++;// register this missile as active for the launcher


	return pBolt;
}

void CCrossbowBolt::Spawn( )
{
	Precache( );
	pev->movetype = MOVETYPE_FLY;
        pev->solid = SOLID_BBOX;

        pev->gravity = 0.1;

        SET_MODEL(ENT(pev), "models/ball.mdl");

	UTIL_SetOrigin( pev, pev->origin );
        //bc UTIL_SetSize(pev, Vector(0, 0, 0), Vector(0, 0, 0));
        UTIL_SetSize(pev, Vector(-16, -16, -16), Vector(16, 16, 16));

        // SOCCERBALL OF DESTRUCTION
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );
		WRITE_BYTE( TE_BEAMFOLLOW );
		WRITE_SHORT(entindex());	// entity
		WRITE_SHORT(m_iTrail );	// model
                WRITE_BYTE( 2 ); // life
                WRITE_BYTE( 6 );  // width
                WRITE_BYTE( 255 );   // r, g, b
                WRITE_BYTE( 255 );   // r, g, b
                WRITE_BYTE( 0 );   // r, g, b
                WRITE_BYTE( 255 );      // brightness
	MESSAGE_END();  // move PHS/PVS data sending into here (SEND_ALL, SEND_PVS, SEND_PHS)



	SetTouch( &CCrossbowBolt::BoltTouch );
	SetThink( &CCrossbowBolt::BubbleThink );
	pev->nextthink = gpGlobals->time + 0.2;
}


void CCrossbowBolt::Precache( )
{
        PRECACHE_MODEL ("models/ball.mdl");
        m_iTrail = PRECACHE_MODEL("sprites/smoke.spr");
}


int	CCrossbowBolt :: Classify ( void )
{
	return	CLASS_NONE;
}

void CCrossbowBolt::BoltTouch( CBaseEntity *pOther )
{
        //SetTouch( NULL );
	SetThink( NULL );
        //pev->solid = SOLID_NOT;

        entvars_t       *pevOwner;
        pevOwner = VARS( pev->owner );


	if (pOther->pev->takedamage)
	{
                //bc a rare RIDICULOUSLY POWERFUL soccerball attack
                if (!(RANDOM_LONG(0,99)) || (pevOwner->health < 9))//supercharged!
                {
                EMIT_SOUND_DYN(ENT(pev), CHAN_WEAPON, "zroar1.wav", 1.0, ATTN_NORM, 0, 100);
                RadiusDamage( pOther->pev->origin,  pev, pev, 256, 512, CLASS_NONE, DMG_BLAST );
		MESSAGE_BEGIN( MSG_PVS, SVC_TEMPENTITY, pev->origin );
			WRITE_BYTE( TE_BEAMCYLINDER );
			WRITE_COORD( pev->origin.x);
			WRITE_COORD( pev->origin.y);
			WRITE_COORD( pev->origin.z);
			WRITE_COORD( pev->origin.x);
			WRITE_COORD( pev->origin.y);
                        WRITE_COORD( pev->origin.z + 768 ); // reach damage radius over .2 seconds
                        WRITE_SHORT( g_sModelIndexWExplosion );
			WRITE_BYTE( 0 ); // startframe
			WRITE_BYTE( 0 ); // framerate
                        WRITE_BYTE( 3 ); // life
			WRITE_BYTE( 32 );  // width
			WRITE_BYTE( 0 );   // noise
                        WRITE_BYTE( 255 );   // r, g, b
                        WRITE_BYTE( 128 );   // r, g, b
                        WRITE_BYTE( 0 );   // r, g, b
                        WRITE_BYTE( 255 ); // brightness
                        WRITE_BYTE( 0 );                // speed
		MESSAGE_END();
                }


                TraceResult tr = UTIL_GetGlobalTrace( );
                entvars_t       *pevOwner;
                pevOwner = VARS( pev->owner );

		// UNDONE: this needs to call TraceAttack instead
		ClearMultiDamage( );

		if ( pOther->IsPlayer() )
		{
			pOther->TraceAttack(pevOwner, gSkillData.plrDmgCrossbowClient, pev->velocity.Normalize(), &tr, DMG_NEVERGIB ); 
		}
		else
		{
			pOther->TraceAttack(pevOwner, gSkillData.plrDmgCrossbowMonster, pev->velocity.Normalize(), &tr, DMG_BULLET | DMG_NEVERGIB ); 
		}

		ApplyMultiDamage( pev, pevOwner );

	}

        SetThink( &CCrossbowBolt::ReturnThink );
        UTIL_SetSize(pev, Vector(0, 0, 0), Vector(0, 0, 0));
        pev->nextthink = gpGlobals->time + 0.1;
}


void CCrossbowBolt::ReturnThink( void )
{
        //pev->solid = SOLID_NOT;
        

        entvars_t       *pevOwner;
        pevOwner = VARS( pev->owner );

        //Vector vecTarget;
	Vector vecDir;

        vecDir = (pevOwner->origin) - (pev->origin);
        vecDir = vecDir.Normalize();
        //vecTarget = vecDir;

        //pev->angles = UTIL_VecToAngles( vecTarget );
        pev->angles = UTIL_VecToAngles( vecDir );

	// this acceleration and turning math is totally wrong, but it seems to respond well so don't change it.
	float flSpeed = pev->velocity.Length();
	{
                //pev->velocity = pev->velocity * 0.2 + vecTarget * (flSpeed * 0.8 + 400);
                pev->velocity = pev->velocity * 0.2 + vecDir * (flSpeed * 0.8 + 400);

                if (pev->velocity.Length() > 2000)
                        pev->velocity = pev->velocity.Normalize() * 2000;
	}

        //bc
        // this is my returning-code.
        // calculate the distance between the ball and player.
        // if distance is less than XX, then kill ball.
        Vector vecDist = pevOwner->origin - pev->origin;
        if ( (vecDist.Length() < 128 ) ||  ( DROP_TO_FLOOR(ENT(pev)) == 0))
        {
                //register ball has returned to player!
                m_pLauncher->m_cActiveRockets--;
                UTIL_Remove( this );
                //SetThink( &CCrossbowBolt::SUB_Remove );
                //pev->nextthink = gpGlobals->time + 0.01;
        }


        pev->nextthink = gpGlobals->time + 0.1;

/*
        CBaseEntity *pOther = NULL;
	Vector vecTarget;
	Vector vecDir;
	float flDist, flMax, flDot;
	TraceResult tr;


	UTIL_MakeAimVectors( pev->angles );
        //vecTarget = gpGlobals->v_forward;
        vecTarget = gpGlobals->v_up;

	flMax = 4096;

        //by ghoul, planethalflife/hlprogramming
        //while ((pOther = UTIL_FindEntityInSphere(pOther, pev->origin,512)) != NULL)
        while ((pOther = UTIL_FindEntityInSphere(pOther, pev->origin,512)) == pevOwner)
        {
                if ((pOther->pev->takedamage != DAMAGE_NO) && (pOther->pev->deadflag != DEAD_DEAD))
                {
                        UTIL_TraceLine(pev->origin,pOther->pev->origin,dont_ignore_monsters,ENT(pev),&tr);
                        if (tr.flFraction >= 0.95)
                        {
                                vecDir = (pOther->Center()) - pev->origin;
                                flDist = vecDir.Length();
                                vecDir = vecDir.Normalize();
                                flDot = DotProduct(gpGlobals->v_forward, vecDir);
                                if ((flDot > 0) && (flDist * (1 - flDot) < flMax))
                                {
                                        flMax = flDist * (1 - flDot);
                                        vecTarget = vecDir;
                                }
                        }
                }
        }

        pev->angles = UTIL_VecToAngles( vecTarget );

	// this acceleration and turning math is totally wrong, but it seems to respond well so don't change it.
	float flSpeed = pev->velocity.Length();
        //if (gpGlobals->time - m_flIgniteTime < 1.0)
	{
		pev->velocity = pev->velocity * 0.2 + vecTarget * (flSpeed * 0.8 + 400);

                if (pev->velocity.Length() > 2000)
                        pev->velocity = pev->velocity.Normalize() * 2000;
	}
        //else
        {
                
                //if (pev->effects & EF_LIGHT)
                //{
                //        pev->effects = 0;
                //        STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/rocket1.wav" );
                //}
                

                //pev->velocity = pev->velocity * 0.2 + vecTarget * flSpeed * 0.798;
                
                //if (pev->waterlevel == 0 && pev->velocity.Length() < 1500)
                //{
                //        Detonate( );
                //}
	}
	// ALERT( at_console, "%.0f\n", flSpeed );
        

        pev->nextthink = gpGlobals->time + 0.1;
*/
}


void CCrossbowBolt::BubbleThink( void )
{
	pev->nextthink = gpGlobals->time + 0.1;

	if (pev->waterlevel == 0)
		return;

	UTIL_BubbleTrail( pev->origin - pev->velocity * 0.1, pev->origin, 1 );
}

#endif

enum crossbow_e {
	CROSSBOW_IDLE1 = 0,	// full
	CROSSBOW_IDLE2,		// empty
	CROSSBOW_FIDGET1,	// full
	CROSSBOW_FIDGET2,	// empty
	CROSSBOW_FIRE1,		// full
	CROSSBOW_FIRE2,		// reload
	CROSSBOW_FIRE3,		// empty
	CROSSBOW_RELOAD,	// from empty
	CROSSBOW_DRAW1,		// full
	CROSSBOW_DRAW2,		// empty
	CROSSBOW_HOLSTER1,	// full
	CROSSBOW_HOLSTER2,	// empty
};

LINK_ENTITY_TO_CLASS( weapon_crossbow, CCrossbow );

void CCrossbow::Spawn( )
{
	Precache( );
	m_iId = WEAPON_CROSSBOW;
        SET_MODEL(ENT(pev), "models/w_soccer.mdl");

	m_iDefaultAmmo = CROSSBOW_DEFAULT_GIVE;

	FallInit();// get ready to fall down.
}

int CCrossbow::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

void CCrossbow::Precache( void )
{
        PRECACHE_MODEL("models/w_soccer.mdl");
        //PRECACHE_MODEL("models/v_crossbow.mdl");
        //PRECACHE_MODEL("models/p_crossbow.mdl");

        //PRECACHE_SOUND("weapons/xbow_fire1.wav");
        //PRECACHE_SOUND("weapons/xbow_reload1.wav");

        PRECACHE_SOUND("zroar1.wav");

/*
        PRECACHE_SOUND("weapons/g_bounce1.wav");
        PRECACHE_SOUND("weapons/g_bounce2.wav");
        PRECACHE_SOUND("weapons/g_bounce3.wav");
        PRECACHE_SOUND("weapons/g_bounce4.wav");
        PRECACHE_SOUND("weapons/g_bounce5.wav");
*/
	UTIL_PrecacheOther( "crossbow_bolt" );

        m_usCrossbow = PRECACHE_EVENT( 1, "events/crossbow1.sc" );

}


int CCrossbow::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "bolts";
	p->iMaxAmmo1 = BOLT_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = CROSSBOW_MAX_CLIP;
	p->iSlot = 2;
	p->iPosition = 2;
	p->iId = WEAPON_CROSSBOW;
	p->iFlags = 0;
	p->iWeight = CROSSBOW_WEIGHT;
	return 1;
}


BOOL CCrossbow::Deploy( )
{
        return DefaultDeploy( "models/null.mdl", "models/ball.mdl", CROSSBOW_DRAW1, "bow" );
}

void CCrossbow::Holster( int skiplocal /* = 0 */ )
{
	m_fInReload = FALSE;// cancel any reload in progress.


	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;
	if (m_iClip)
		SendWeaponAnim( CROSSBOW_HOLSTER1 );
	else
		SendWeaponAnim( CROSSBOW_HOLSTER2 );
}

void CCrossbow::PrimaryAttack( void )
{
        if ( !m_cActiveRockets )
                FireBolt();
}


void CCrossbow::FireBolt()
{
	TraceResult tr;

	if (m_iClip == 0)
	{
		PlayEmptySound( );
		return;
	}

	m_pPlayer->m_iWeaponVolume = QUIET_GUN_VOLUME;

        //m_iClip--;

	int flags;
#if defined( CLIENT_WEAPONS )
	flags = FEV_NOTHOST;
#else
	flags = 0;
#endif

	PLAYBACK_EVENT_FULL( flags, m_pPlayer->edict(), m_usCrossbow, 0.0, (float *)&g_vecZero, (float *)&g_vecZero, 0, 0, m_iClip, m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType], 0, 0 );

	// player "shoot" animation
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	Vector anglesAim = m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle;
	UTIL_MakeVectors( anglesAim );
	
	anglesAim.x		= -anglesAim.x;
        Vector vecSrc    = m_pPlayer->GetGunPosition( ) + (gpGlobals->v_up * -16);
	Vector vecDir	 = gpGlobals->v_forward;

#ifndef CLIENT_DLL
        CCrossbowBolt *pBolt = CCrossbowBolt::BoltCreate( this );
	pBolt->pev->origin = vecSrc;
	pBolt->pev->angles = anglesAim;
	pBolt->pev->owner = m_pPlayer->edict();
        pBolt->pev->velocity = vecDir * BOLT_AIR_VELOCITY;
        pBolt->pev->speed = BOLT_AIR_VELOCITY;



        //pBolt->pev->avelocity.z = 10;
#endif

  //      if (!m_iClip && m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		// HEV suit - indicate out of ammo condition
    //            m_pPlayer->SetSuitUpdate("!HEV_AMO0", FALSE, 0);

        m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.2;


//                m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 1;
}


void CCrossbow::SecondaryAttack()
{
}


void CCrossbow::Reload( void )
{
}


void CCrossbow::WeaponIdle( void )
{
}



class CCrossbowAmmo : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_crossbow_clip.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_crossbow_clip.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		if (pOther->GiveAmmo( AMMO_CROSSBOWCLIP_GIVE, "bolts", BOLT_MAX_CARRY ) != -1)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
			return TRUE;
		}
		return FALSE;
	}
};
LINK_ENTITY_TO_CLASS( ammo_crossbow, CCrossbowAmmo );



#endif
