/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"


#define	HANDGRENADE_PRIMARY_VOLUME		450

enum handgrenade_e {
	HANDGRENADE_IDLE = 0,
	HANDGRENADE_FIDGET,
	HANDGRENADE_PINPULL,
	HANDGRENADE_THROW1,	// toss
	HANDGRENADE_THROW2,	// medium
	HANDGRENADE_THROW3,	// hard
	HANDGRENADE_HOLSTER,
	HANDGRENADE_DRAW
};


LINK_ENTITY_TO_CLASS( weapon_handgrenade, CHandGrenade );


void CHandGrenade::Spawn( )
{
	Precache( );
	m_iId = WEAPON_HANDGRENADE;
        SET_MODEL(ENT(pev), "models/w_chicken.mdl");

#ifndef CLIENT_DLL
	pev->dmg = gSkillData.plrDmgHandGrenade;
#endif

	m_iDefaultAmmo = HANDGRENADE_DEFAULT_GIVE;

	FallInit();// get ready to fall down.
}


void CHandGrenade::Precache( void )
{
        PRECACHE_MODEL("models/w_chicken.mdl");

	PRECACHE_MODEL("models/p_grenade.mdl");
        PRECACHE_MODEL("models/xhair1.mdl");

        PRECACHE_SOUND("bok.wav");
}

int CHandGrenade::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "Hand Grenade";
	p->iMaxAmmo1 = HANDGRENADE_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 4;
	p->iPosition = 0;
	p->iId = m_iId = WEAPON_HANDGRENADE;
	p->iWeight = HANDGRENADE_WEIGHT;
        //p->iFlags = ITEM_FLAG_LIMITINWORLD | ITEM_FLAG_EXHAUSTIBLE;
        p->iFlags = 0;

	return 1;
}


BOOL CHandGrenade::Deploy( )
{
        return DefaultDeploy( "models/null.mdl", "models/p_grenade.mdl", HANDGRENADE_DRAW, "crowbar" );
}


BOOL CHandGrenade::CanHolster( void )
{
}

void CHandGrenade::Holster( int skiplocal /* = 0 */ )
{
}



void CHandGrenade::PrimaryAttack()
{
//bc
        UTIL_MakeVectors ( m_pPlayer->pev->v_angle );
        TraceResult tr;

        //find room in front of player
        UTIL_TraceLine ( m_pPlayer->pev->origin, m_pPlayer->pev->origin + (gpGlobals->v_forward * 256), ignore_monsters, ENT(pev), &tr );

        //pull out of wall a bit & raise altitude very high
        Vector vecSrc = tr.vecEndPos + (gpGlobals->v_forward * -2) + (gpGlobals->v_up * 1024);


        //bakok sound fx
        EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "bok.wav", 1.0, ATTN_NORM);




        //find floor.
        UTIL_TraceLine ( vecSrc, vecSrc - Vector( 0, 0, 2048 ), ignore_monsters, ENT(pev), &tr );

        CSprite *m_pHair;
        m_pHair = CSprite::SpriteCreate( "models/xhair1.mdl", tr.vecEndPos, FALSE );
        if (m_pHair)  //???
        {
                m_pHair->pev->avelocity.y =  160; //spinning animation
                m_pHair->SetThink( &CHandGrenade::SUB_Remove );
                m_pHair->pev->nextthink = gpGlobals->time + 2.3;//lifespan
        }

        CGrenade::ShootContact( m_pPlayer->pev, vecSrc, Vector(0,0,-8));


       m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 2.0;
}


void CHandGrenade::WeaponIdle( void )
{
}




